/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	SystemClock_Config.c		The function to initialize MCU clocks

	When		Who	What
	21/01/20	ac	created
	05/07/21	ac	Remove LSE and RTC enable

----------------------------------------------------------------------
*/

#include	"aa.h"		// For BSP_DEBUG_BKPT

#include	"stm32f1xx_ll_rcc.h"
#include	"stm32f1xx_ll_system.h"
#include	"stm32f1xx_ll_pwr.h"
#include	"stm32f1xx_ll_bus.h"

static	void   Error_Handler	(void) ;

// ----------------------------------------------------------------------------
//	SystemClock_Config() may be generated by STM32CubeMX to configure the clock in accordance with your needs
//	You can use CubeMX to generate the LL code, and copy SystemClock_Config() here.

//	This function must:
//		Not initialize SysTick
//		Initialize peripheral clocks
//		Set TIMPRE if requested
//		LSE and RTC initialization are complex, it is better not to do it here

//	Use HSE bypass as system clock source, system clock is 72 MHz
//	LSE and RTC enabled
//	MCO output enabled
//	ADC clock configured

void SystemClock_Config (void)
{
	LL_FLASH_SetLatency(LL_FLASH_LATENCY_2);

	if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_2)
	{
		Error_Handler();
	}

	/* Enable HSE */
	LL_RCC_HSE_EnableBypass();
	LL_RCC_HSE_Enable();
	while(LL_RCC_HSE_IsReady() != 1)
	{
	}

	/* Configure PLL */
	LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE_DIV_1, LL_RCC_PLL_MUL_9);
	LL_RCC_PLL_Enable();
	while(LL_RCC_PLL_IsReady() != 1)
	{
	}
	LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
	LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_2);
	LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_1);
	LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);
	while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
	{
	}

	/* Configure MCO source */
	LL_RCC_ConfigMCO(LL_RCC_MCO1SOURCE_PLLCLK_DIV_2);

	/* Configure ADC clock */
	LL_RCC_SetADCClockSource(LL_RCC_ADC_CLKSRC_PCLK2_DIV_6);
}

// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if SystemClock_Config() configure LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_AFIO) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	// Disable JTAG and enable SWJ
	AFIO->MAPR &= ~AFIO_MAPR_SWJ_CFG ;
	AFIO->MAPR |= AFIO_MAPR_SWJ_CFG_JTAGDISABLE ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

// ----------------------------------------------------------------------------
//	This function is used by CubeMX generated code

BSP_ATTR_USED static	void   Error_Handler	(void)
{
	BSP_DEBUG_BKPT () ;
	while (1)
	{
	}
}

// ----------------------------------------------------------------------------
