/*
----------------------------------------------------------------------
	
	Alain Chebrou

	Spartan RT kernel

	bspcfg.h	Target BSP specific configuration used by kernel
				WIN32 port

	When		Who	What
	30/12/13	ac	Creation

----------------------------------------------------------------------
*/

#if ! defined AA_BSPCFG_H_
#define AA_BSPCFG_H_
//--------------------------------------------------------------------------------
//	BSP configuration

#define	BSP_TICK_RATE		20					// Tick rate Hz

#define	BSP_MAX_INT_PRIO	0xB0				// Max usable interrupt priority 

//--------------------------------------------------------------------------------
//	Stack management

#define	bspSTACK_GROWS_DOWN	1					// 1 of stack grows from high to low addresses
#define	bspSTACK_PATTERN	0xDEADBEEFu			// Fill the stack with this value for stack usage check

//--------------------------------------------------------------------------------
// To manage task selector accelerator bitmaps

typedef	uint32_t			bspPrio_t ;			// Type used for priority accelerator bitmaps

#define	bspPRIO_POW			5					// log2 (sizeof(bspPrio_t)*8), eg uint32_t => 5 so 1<<5 = 32
#define	bspPRIO_SIZE		(1 << bspPRIO_POW)	// Count of bits in bspPrio_t
#define	bspPRIO_MASK		(bspPRIO_SIZE - 1)	// uint16_t => 0x0F, uint32_t => 0x1F

//--------------------------------------------------------------------------------
//	Heap simulation

#define	BSP_HEAP_SIZE		65000  //  0x4000
extern	uint8_t				bspHeap [BSP_HEAP_SIZE] ;
#define	_Heap_Begin			(bspHeap [0])
#define	_Heap_Limit			(bspHeap [BSP_HEAP_SIZE])

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	BSP functions for kernel use only

#ifdef __cplusplus
extern "C" {
#endif

//--------------------------------------------------------------------------------
//	Return most significant bit position of word
//	1 => 0, 3 => 1, 0x7FFFFFFF => 30
//	word 0 should return value >= bspPRIO_SIZE

static _inline uint32_t	bspMsbPos		(uint32_t word)
{
	uint32_t index;
//printf ("bspMsbPos %08X %d\n", word, _BitScanReverse(& index, word) ? index : -1) ;
	_BitScanReverse(& index, word) ? index : bspPRIO_SIZE;
	AA_ASSERT (index < bspPRIO_SIZE) ;
	return index ;
}

// Insert a NOP instruction, useless for WIN32 port
static	_inline	void bspNop (void)
{
	return ;
}

typedef struct aaTcb_t aaTcb_t ;

void			aaMain					(void) ;
	
void			bspInit					(void) ;
void			bspStart				(void) ;
void			bspTaskInit				(aaTcb_t * pTcb, aaTaskFunction pEntry, uintptr_t arg) ;
uint32_t		bspGetTimerDelta		(void) ;

void			aaCriticalEnter			(void) ;
void			aaCriticalExit			(void) ;

void			bspEnableIrq			(void) ;
void			bspDisableIrq			(void) ;

void			bspTaskSwitch			(void) ;
void			bspTaskSwitchFromIsr	(void) ;


#ifdef __cplusplus
}
#endif
//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
#endif	// AA_BSPCFG_H_
