/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	itm.h		ARM SWO trace

	When		Who	What
	11/07/17	ac	Creation
	01/11/18	ac	Add swoIsEnabled()

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined AA_SWO_H_
#define AA_SWO_H_
//-----------------------------------------------------------------------------

#include	"bspcfg.h"		// for BSP_SWO_STIMULUS

void		swoInit			(uint32_t portBits, uint32_t cpuCoreFreqHz, uint32_t baudrate) ;
uint32_t	swoIsEnabled	(void) ;

void		swoSend8		(uint8_t  value, uint8_t portNo) ;
void		swoSend16		(uint16_t value, uint8_t portNo) ;
void		swoSend32		(uint32_t value, uint8_t portNo) ;

void		swoPut			(uintptr_t dev, char value) ;	// Send to ITM stimulus BSP_SWO_STIMULUS

void		swoPutStr		(char * pStr) ;					// Send a 0 terminated string to stimulus port BSP_SWO_STIMULUS
void		swoPutCharFn	(char value, uintptr_t arg) ;	// To be used with aaPrintfEx() to print to SWO port BSP_SWO_STIMULUS

// Send 1 char to ITM stimulus port BSP_SWO_STIMULUS
BSP_ATTR_ALLWAYS_INLINE void 		swoPutChar 		(char value)
{
	swoSend8 ((uint8_t) value, BSP_SWO_STIMULUS) ;
}

//-----------------------------------------------------------------------------
#endif 	// AA_SWO_H_
