
/*--------------------------------------------------------------------------
/*
 * Memory Spaces Definitions.
 *
 * Update for a specific board. 
 *   FLASH.ORIGIN : starting address of flash
 *   FLASH.LENGTH : length of flash
 *   RAM.ORIGIN   : starting address of main RAM
 *   RAM.LENGTH   : length of main RAM
 *
 * The values below can be addressed in linker scripts
 * using functions like 'ORIGIN(RAM)' or 'LENGTH(RAM)'.
 */

/* Memory definitions for  STM32F103xx
 * Adjust your amount of FLASH and RAM
 *
 *	F103	FLASH	RAM		Define
 *	x4		16		6		STM32F103x6
 *	x6		32		10		STM32F103x6
 *	x8		64		20		STM32F103xB
 *	xB		128		20		STM32F103xB
 *	xC		256		48		STM32F103xE
 *	xD		384		64		STM32F103xE
 *	xE		512		64		STM32F103xE
 *
 * Can't execute code from SRAM
 * No CCRRAM or xTCRAM, no BKPRAM
 * FLASHB2 only for XL-density devices
 * EXTMEMBx only if FSMC is present
 */

MEMORY
{
	FLASH         (rx) : ORIGIN = 0x08000000, LENGTH = 128K
	FLASHB2       (rx) : ORIGIN = 0x00000000, LENGTH = 0
	RAM           (rw) : ORIGIN = 0x20000000, LENGTH = 20K

	EXTMEMB0      (rx) : ORIGIN = 0x00000000, LENGTH = 0
	EXTMEMB1      (rx) : ORIGIN = 0x00000000, LENGTH = 0
	EXTMEMB2      (rx) : ORIGIN = 0x00000000, LENGTH = 0
	EXTMEMB3      (rx) : ORIGIN = 0x00000000, LENGTH = 0
}

INCLUDE ../sections.ld