/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dacbasic.c	DAC management with DMA

	When		Who	What
	05/05/18	ac	Creation

----------------------------------------------------------------------
*/
#include	"aa.h"
#include	"aakernel.h"
#include	"gpiobasic.h"
#include	"dacbasic.h"
#include	"aalogmes.h"

//----------------------------------------------------------------------

typedef struct dacDesc_t
{
	uint16_t		id ;			// Descriptor id
	uint8_t			periphRstPos ;	// DAC periph reset bit pos
	uint8_t			periphEnPos ;	// DAC periph enable bit pos
	DAC_TypeDef		* pDac ;		// Pointer to DAC registers
	gpioPinDesc_t	dacOut1 ;		// DAC channel 1 output pin descriptor
	gpioPinDesc_t	dacOut2 ;		// DAC channel 1 output pin descriptor
	dmaDesc_t		dma1 ;
	dmaDesc_t		dma2 ;

} dacDesc_t ;

// Include the proper DAC descriptors
#if defined STM32F429xx
//#include "dacF429ZI.h"
#include	"stm32f4xx_ll_gpio.h"	// for LL_GPIO_AF_x
#endif

static	const dacDesc_t	dacDescArray [] =
{
	// For 2 independent channels (2 DMA)
	#if defined WITH_DAC_1 && (WITH_DAC_1 == 1)
	{
		10,								// id
		RCC_APB1RSTR_DACRST_Pos,		// DAC periph reset bit pos
		RCC_APB1ENR_DACEN_Pos,			// DAC periph enable bit pos
		DAC1,							// Pointer to DAC registers
		{	'A',	4,	LL_GPIO_AF_0,	GPIO_MODE_ANALOG },		// Channel 1
		{	'A',	5,	LL_GPIO_AF_0,	GPIO_MODE_ANALOG },		// channel 2
		{	// DMA channel 1 or DUAL
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TOSTREAMCHAN (5, 7),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			DAC_INT_PRIORITY,					//	Interrupt priority
		},
		{	// DMA channel 2
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TOSTREAMCHAN (6, 7),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			DAC_INT_PRIORITY,					//	Interrupt priority
		},
	},
	#endif	// WITH_DAC_1

	// For channel 1 only or DUAL
	#if defined WITH_DAC_1 && (WITH_DAC_1 == 1)
	{
		11,								// id
		RCC_APB1RSTR_DACRST_Pos,		// DAC periph reset bit pos
		RCC_APB1ENR_DACEN_Pos,			// DAC periph enable bit pos
		DAC1,							// Pointer to DAC registers
		{	'A',	4,	LL_GPIO_AF_0,	GPIO_MODE_ANALOG },		// Channel 1
		{	'A',	5,	LL_GPIO_AF_0,	GPIO_MODE_ANALOG },		// channel 2
		{	// DMA channel 1 or DUAL
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TOSTREAMCHAN (5, 7),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			DAC_INT_PRIORITY,					//	Interrupt priority
		},
		{	// DMA channel 2
			DMA_NONE,							//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TOSTREAMCHAN (6, 7),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			DAC_INT_PRIORITY,					//	Interrupt priority
		},
	},
	#endif	// WITH_DAC_1

	// For channel 2 only
	#if defined WITH_DAC_1 && (WITH_DAC_1 == 1)
	{
		12,								// id
		RCC_APB1RSTR_DACRST_Pos,		// DAC periph reset bit pos
		RCC_APB1ENR_DACEN_Pos,			// DAC periph enable bit pos
		DAC1,							// Pointer to DAC registers
		{	'A',	4,	LL_GPIO_AF_0,	GPIO_MODE_ANALOG },		// Channel 1
		{	'A',	5,	LL_GPIO_AF_0,	GPIO_MODE_ANALOG },		// channel 2
		{	// DMA channel 1 or DUAL
			DMA_NONE,							//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TOSTREAMCHAN (5, 7),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			DAC_INT_PRIORITY,					//	Interrupt priority
		},
		{	// DMA channel 2
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TOSTREAMCHAN (6, 7),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			DAC_INT_PRIORITY,					//	Interrupt priority
		},
	},
	#endif	// WITH_DAC_1
} ;

// Count of DAC devices in dacDescArray
STATIC	const uint32_t	dacArrayCount = sizeof (dacDescArray) / sizeof (dacDesc_t) ;

//----------------------------------------------------------------------
//	DAC I/O management descriptor

typedef struct
{
	const dacDesc_t	* pDacDesc ;	// DAC descriptor
	uint32_t		flags ;			// Common configuration flags
	uint32_t		flagsChan [2] ;	// Channels configuration flags
	uint32_t		dataReg [2] ;	// Data register offset (DMA_SxPAR)
	dmaCallback_t	cb [2] ;		// DAC callback
	uintptr_t		cbArg [2] ; 	// DAC callback arg

} dacIoDesc_t ;

STATIC	dacIoDesc_t	ioDesc1 ;	// IO Descriptor for DAC1
STATIC	dacIoDesc_t	* pIoDesc = & ioDesc1 ;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Initialize DAC common parameters
//	Flags is DAC_DUAL or a combination of DAC_MONO1 and DAC_MONO2

dacStatus_t 	dacInit			(uint32_t flags, uint32_t id)
{
	const dacDesc_t	* pDacDesc;
	DAC_TypeDef		* pDac ;
	uint32_t		ii ;

	// Find DAC descriptor from id
	for (ii = 0 ; ii < dacArrayCount ; ii++)
	{
		if (dacDescArray [ii].id == id)
		{
			break ;
		}
	}
	if (ii == dacArrayCount)
	{
		return DAC_EFAIL ;	// DAC descriptor not found
	}
	pDacDesc = & dacDescArray [ii] ;
	pDac = pDacDesc->pDac ;

	// Set DAC IO descriptor
	pIoDesc->pDacDesc = pDacDesc ;
	pIoDesc->flags    = flags ;

	// Enable the peripheral clock for DAC
	rccEnableClock (pDac, 1u << pDacDesc->periphEnPos) ;

	// Reset peripheral
	rccResetPeriph (pDac, 1u << pDacDesc->periphRstPos) ;

	//	Configure output pins
	if ((flags & (DAC_MONO1 | DAC_DUAL)) != 0)
	{
		gpioConfigurePin (& pDacDesc->dacOut1) ;
	}
	if ((flags & (DAC_MONO2 | DAC_DUAL)) != 0)
	{
		gpioConfigurePin (& pDacDesc->dacOut2) ;
	}

	return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Initialize one channel parameters.
//	flags is a combination of:
//		DAC_OUTBUFFDIS
//		DAC_OUTBUFFEN
//		DAC_BITS8
//		DAC_BITS12
//		DAC_ALIGNLEFT
//		DAC_ALIGNRIGHT
//	trigSrc is one of LL_DAC_TRIG_xxx:
//		LL_DAC_TRIG_SOFTWARE
//		LL_DAC_TRIG_EXT_TIM2_TRGO
//		LL_DAC_TRIG_EXT_TIM8_TRGO
//		LL_DAC_TRIG_EXT_TIM4_TRGO
//		LL_DAC_TRIG_EXT_TIM6_TRGO
//		LL_DAC_TRIG_EXT_TIM7_TRGO
//		LL_DAC_TRIG_EXT_TIM5_TRGO
//		LL_DAC_TRIG_EXT_EXTI_LINE9

dacStatus_t 	dacSetChannel		(uint32_t channel, uint32_t flags, uint32_t trigSrc)
{
	uint32_t	llChannel ;
	uint32_t	reg, ix ;
	DAC_TypeDef	* pDac = pIoDesc->pDacDesc->pDac ;

	// Memorize configuration
	if (channel == DAC_CHAN1)
	{
		llChannel = LL_DAC_CHANNEL_1 ;
		pIoDesc->flagsChan [0] = flags ;
	}
	else
	{
		llChannel = LL_DAC_CHANNEL_2 ;
		pIoDesc->flagsChan [1] = flags ;
	}

	// Compute DAC data register address for DMA_SxPAR register
	reg = 2 ;
	ix = 0 ;
	if ((pIoDesc->flags & DAC_DUAL) != 0)
	{
		reg = 8 ;
	}
	else if (channel ==  DAC_CHAN2)
	{
		reg = 5 ;
		ix = 1 ;
	}
	else
	{
	}

	if ((pIoDesc->flagsChan [ix] & DAC_BITS8) != 0)
	{
		reg += 2 ;
	}
	else if ((pIoDesc->flagsChan [ix] & DAC_ALIGNLEFT) != 0)
	{
		reg += 1 ;
	}
	else
	{
	}

	pIoDesc->dataReg [channel-1] = (uint32_t) pDac + (reg * 4) ;

	// Set the output buffer for the selected DAC channel (default is enabled)
	if ((flags & DAC_OUTBUFFDIS) != 0)
	{
		LL_DAC_SetOutputBuffer (pDac, llChannel, LL_DAC_OUTPUT_BUFFER_DISABLE) ;
	}

	// Set trigger source
	LL_DAC_SetTriggerSource (pDac, llChannel, trigSrc) ;

	return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Set callback for one independent channel or the dual channel
//	DMA and DAC interrupt callback

dacStatus_t 	dacSetCallback		(uint32_t channel, dmaCallback_t cb, uintptr_t arg)
{
	if (channel == DAC_CHAN1)
	{
		pIoDesc->cb[0]     = cb ;
		pIoDesc->cbArg [0] = arg ;
		if (pIoDesc->pDacDesc->dma1.dmaNum != DMA_NONE)
		{
			dmaSetCallback	(& pIoDesc->pDacDesc->dma1, cb, arg) ;
		}
	}
	else if (channel == DAC_CHAN2)
	{
		pIoDesc->cb[1]     = cb ;
		pIoDesc->cbArg [1] = arg ;
		if (pIoDesc->pDacDesc->dma2.dmaNum != DMA_NONE)
		{
			dmaSetCallback	(& pIoDesc->pDacDesc->dma2, cb, arg) ;
		}
	}
	else
	{
		return DAC_EARG ;
	}

	return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Set DMA callback.
//	channel : one of DAC_CHAN1 or DAC_CHAN2
//	flags   : one of DAC_DMA_xxx and optional DAC_DMA_NOIT
//	length  : count of items in DMA buffers

dacStatus_t 	dacDmaInit		(uint32_t channel, uint32_t flags, uint32_t length)
{
	const dmaDesc_t 	* pDmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			llChannel ;
	uint32_t			streamNum ;
	DAC_TypeDef			* pDac = pIoDesc->pDacDesc->pDac ;
	uint32_t			reg ;

	if (channel == DAC_CHAN1)
	{
		llChannel = LL_DAC_CHANNEL_1 ;
		pDmaDesc  = & pIoDesc->pDacDesc->dma1 ;
		pIoDesc->flagsChan [0] |= flags & DAC_DMA_MASK ;
	}
	else
	{
		llChannel = LL_DAC_CHANNEL_2 ;
		pDmaDesc  = & pIoDesc->pDacDesc->dma2 ;
		pIoDesc->flagsChan [1] |= flags & DAC_DMA_MASK ;
	}
	if (pDmaDesc->dmaNum == DMA_NONE)
	{
		AA_ASSERT (0) ;
		return DAC_ESTATE ;		// No DMA descriptor
	}
	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	streamNum	= DMA_GETSTREAM (pDmaDesc) ;

	// Enable DMA clock
	rccEnableClock (pDma, 1 << ((pDma == DMA1) ? RCC_AHB1ENR_DMA1EN_Pos : RCC_AHB1ENR_DMA2EN_Pos)) ;

	// Reset stream
	dmaStreamAddress (pDma, streamNum)->CR = 0 ;

	LL_DMA_SetChannelSelection (pDma, streamNum, DMA_GETCHANNEL (pDmaDesc) << 25);

	// Compute Peripheral data size
	reg = ((pIoDesc->flags & DAC_DUAL) != 0) ? LL_DMA_PDATAALIGN_WORD : ((pIoDesc->flagsChan[channel-1] & DAC_BITS8) != 0) ? LL_DMA_PDATAALIGN_BYTE : LL_DMA_PDATAALIGN_HALFWORD ;

	LL_DMA_ConfigTransfer (pDma, streamNum,
							LL_DMA_DIRECTION_MEMORY_TO_PERIPH |
							(((flags & DAC_DMA_CIRCULAR) != 0) ? LL_DMA_MODE_CIRCULAR : LL_DMA_MODE_NORMAL) |
							LL_DMA_PERIPH_NOINCREMENT         |
							LL_DMA_MEMORY_INCREMENT           |
							reg								  |		// LL_DMA_PDATAALIGN_xxx
							LL_DMA_MDATAALIGN_HALFWORD        |		// direct mode so: MSIZE <= PSIZE
							LL_DMA_PRIORITY_HIGH               );

	// Set double buffer if required
	if ((flags & DAC_DMA_DOUBLE) != 0)
	{
		LL_DMA_EnableDoubleBufferMode (pDma, streamNum) ;
	}

	// Set DMA transfer size
	LL_DMA_SetDataLength (pDma, streamNum, length) ;

	// Configure interrupts
	if ((flags & DAC_DMA_NOIT) == 0)
	{
		//	Configure NVIC for DMA transfer complete/error interrupts
		dmaSetIrq (pDmaDesc) ;

		// Enable DMA transfer interrupt: transfer error
		LL_DMA_EnableIT_TE (pDma, streamNum) ;
		LL_DMA_EnableIT_TC (pDma, streamNum) ;

		// Configure NVIC to enable DAC interruptions
		NVIC_SetPriority (TIM6_DAC_IRQn, DAC_INT_PRIORITY) ;
		NVIC_EnableIRQ   (TIM6_DAC_IRQn) ;

		// Enable DAC channel underrun interrupt
		if (flags & DAC_CHAN1)
		{
			LL_DAC_EnableIT_DMAUDR1 (pDac) ;
		}
		else
		{
			LL_DAC_EnableIT_DMAUDR2 (pDac) ;
		}
	}
	else
	{
		// Disable DMA interrupts
		LL_DMA_DisableIT_TE (pDma, streamNum) ;
		LL_DMA_DisableIT_TC (pDma, streamNum) ;

		if (flags & DAC_CHAN1)
		{
			LL_DAC_DisableIT_DMAUDR1 (pDac) ;
		}
		else
		{
			LL_DAC_DisableIT_DMAUDR2 (pDac) ;
		}
	}

	// Enable DAC channel DMA request
	LL_DAC_EnableDMAReq (pDac, llChannel);

	 return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Set DMA buffers
//	If DAC_DMA_CIRCULAR or DAC_DMA_SINGLE only buffer 1 is used. To be called before dacStart()
//	If DAC_DMA_DOUBLE and called before dacStart(): buffer 1 and buffer 2 are used
//	If DAC_DMA_DOUBLE and called after dacStart(): only buffer 1 is used

dacStatus_t 	dacDmaBuffer	(uint32_t channel, const void * buffer1, const void * buffer2)
{
	const dmaDesc_t 	* pDmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			streamNum ;
	DAC_TypeDef			* pDac = pIoDesc->pDacDesc->pDac ;
	uint32_t			ix ;

	// Find the DMA configuration according to the channel
	if (channel == DAC_CHAN1)
	{
		pDmaDesc = & pIoDesc->pDacDesc->dma1 ;
		ix		 = 0 ;
	}
	else
	{
		pDmaDesc = & pIoDesc->pDacDesc->dma2 ;
		ix		 = 1 ;
	}
	if (pDmaDesc->dmaNum == DMA_NONE)
	{
		AA_ASSERT (0) ;
		return DAC_ESTATE ;
	}
	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	streamNum	= DMA_GETSTREAM (pDmaDesc) ;

	// Set DMA addresses
	if ((pDac->CR & DMA_SxCR_EN) == 0)
	{
		// DAC not enabled: initialize DMA buffers
		LL_DMA_SetMemoryAddress (pDma, streamNum, (uint32_t) buffer1) ;
		LL_DMA_SetPeriphAddress (pDma, streamNum, pIoDesc->dataReg [ix]) ;

		if ((pIoDesc->flagsChan [ix] & DAC_DMA_DOUBLE) != 0)
		{
			// Also set second buffer address
			LL_DMA_SetMemory1Address (pDma, streamNum, (uint32_t) buffer2) ;
		}
	}
	else
	{
		// DAC enabled : update memory DMA buffer not in use (double buffer only)
		if (LL_DMA_GetCurrentTargetMem (pDma, streamNum) == 0)
		{
			LL_DMA_SetMemory1Address (pDma, streamNum, (uint32_t) buffer1) ;
		}
		else
		{
			LL_DMA_SetMemoryAddress (pDma, streamNum, (uint32_t) buffer1) ;
		}
	}

	return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Start DAC generation on specified channel

dacStatus_t 	dacStart		(uint32_t channel)
{
	const dmaDesc_t 	* pDmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			streamNum ;
	DAC_TypeDef			* pDac = pIoDesc->pDacDesc->pDac ;
	uint32_t			llChannel ;
	__IO uint32_t		wait_loop_index ;

	// Find the DMA configuration according to the channel
	if (channel == DAC_CHAN1)
	{
		llChannel = LL_DAC_CHANNEL_1 ;
		pDmaDesc = & pIoDesc->pDacDesc->dma1 ;
	}
	else
	{
		llChannel = LL_DAC_CHANNEL_2 ;
		pDmaDesc = & pIoDesc->pDacDesc->dma2 ;
	}

	// Enable the DMA transfer
	if (pDmaDesc->dmaNum != DMA_NONE)
	{
		pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
		streamNum	= DMA_GETSTREAM (pDmaDesc) ;
		LL_DMA_EnableStream (pDma, streamNum) ;
	}

	// Enable DAC channel
	LL_DAC_Enable (pDac, llChannel) ;
	wait_loop_index = ((LL_DAC_DELAY_STARTUP_VOLTAGE_SETTLING_US * (SystemCoreClock / (100000 * 2))) / 10) ;
	while(wait_loop_index != 0)
	{
		wait_loop_index-- ;
	}

	// Enable DAC channel trigger
	LL_DAC_EnableTrigger (pDac, llChannel) ;

	return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Stop DAC generation on specified channel

dacStatus_t 	dacStop		(uint32_t channel)
{
	const dmaDesc_t 	* pDmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			streamNum ;
	DAC_TypeDef			* pDac = pIoDesc->pDacDesc->pDac ;
	uint32_t			llChannel ;

	// Find the DMA configuration according to the channel
	if (channel == DAC_CHAN1)
	{
		llChannel = LL_DAC_CHANNEL_1 ;
		pDmaDesc = & pIoDesc->pDacDesc->dma1 ;
	}
	else
	{
		llChannel = LL_DAC_CHANNEL_2 ;
		pDmaDesc = & pIoDesc->pDacDesc->dma2 ;
	}
	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	streamNum	= DMA_GETSTREAM (pDmaDesc) ;

	LL_DAC_DisableTrigger	(pDac, llChannel) ;
	LL_DAC_Disable 			(pDac, llChannel) ;
	if(pDmaDesc->dmaNum != DMA_NONE)
	{
		LL_DMA_DisableStream (pDma, streamNum) ;
	}
	return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	Set static output value

dacStatus_t 	dacOuput		(uint32_t channel, uint32_t value)
{
	if (channel == DAC_CHAN1)
	{
		* (uint32_t *) pIoDesc->dataReg [0] = value ;
	}
	else
	{
		* (uint32_t *) pIoDesc->dataReg [1] = value ;
	}
	return DAC_ENONE ;
}

//----------------------------------------------------------------------
// Trig DAC conversion by software

dacStatus_t 	dacSwTrigger	(uint32_t channel)
{
//	LL_DAC_TrigSWConversion (pIoDesc->pDacDesc->pDac, (channel == DAC_CHAN1) ? LL_DAC_CHANNEL_1 : LL_DAC_CHANNEL_2) ;
	LL_DAC_TrigSWConversion (pIoDesc->pDacDesc->pDac, channel) ;

    return DAC_ENONE ;
}

//----------------------------------------------------------------------
//	DAC interrupt handler, called from timer interrupt
//	Under run interrupt

void	dacBasicIrqHandler (void)
{
	// Check whether DAC channel1 underrun caused the DAC interrupt
	if (LL_DAC_IsActiveFlag_DMAUDR1 (DAC1) != 0)
	{
		// Clear flag DAC channel1 underrun
		LL_DAC_ClearFlag_DMAUDR1 (DAC1) ;

	    // Call interrupt treatment function
		if (pIoDesc->cb [0] != NULL)
		{
			(* pIoDesc->cb [0]) (DAC_EV_UNDR, 0, DAC_CHAN1) ;
		}
	}

	// Check whether DAC channel2 underrun caused the DAC interrupt
	if (LL_DAC_IsActiveFlag_DMAUDR2 (DAC1) != 0)
	{
		// Clear flag DAC channel2 underrun
		LL_DAC_ClearFlag_DMAUDR2 (DAC1) ;

	    // Call interrupt treatment function
		if (pIoDesc->cb [1] != NULL)
		{
			(* pIoDesc->cb [1]) (DAC_EV_UNDR, 0, DAC_CHAN2) ;
		}
	}
}

//----------------------------------------------------------------------
