/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dacbasic.h	DAC management with DMA

	When		Who	What
	05/05/18	ac	Creation

----------------------------------------------------------------------
*/
#if ! defined DACBASIC_H_
#define DACBASIC_H_
//----------------------------------------------------------------------

#include	"dmabasic.h"
#include	"stm32f1xx_ll_dac.h"

// To define the supported DAC devices

#define	WITH_DAC_1		1
#define	WITH_DAC_2		0	// WARNING: DAC2 is not supported by this implementation

//----------------------------------------------------------------------
// Initialization and setting flags

#define		DAC_CHAN1			0x00000001		// Channel 1 id
#define		DAC_CHAN2			0x00000002		// Channel 2 id

// For dacInit()
#define		DAC_MONO1			0x00000010		// Use independent channel 1
#define		DAC_MONO2			0x00000020		// Use independent channel 2
#define		DAC_DUAL			0x00000040		// Use dual channel 1 and 2

// For dacSetChannel()
#define		DAC_OUTBUFFDIS		0x00000100		// Disable output buffer
#define		DAC_OUTBUFFEN		0x00000000		// Enable output buffer (default)

#define		DAC_BITS8			0x00000200		// 8 bits words
#define		DAC_BITS12			0x00000000		// 12 bits words (default)

#define		DAC_ALIGNLEFT		0x00000400		// Left aligned data
#define		DAC_ALIGNRIGHT		0x00000000		// Right aligned data (default)

// For dacDmaInit()
#define		DAC_DMA_CIRCULAR	0x00001000		// Circular DMA buffer
#define		DAC_DMA_DOUBLE		0x00002000		// Double DMA buffer
#define		DAC_DMA_SINGLE		0x00004000		// One DMA buffer, not circular
#define		DAC_DMA_MASK		0x00007000

#define		DAC_DMA_NOIT		0x00010000		// No DMA interrupt

//----------------------------------------------------------------------

//	Default priority for DAC interrupt
#define		DAC_INT_PRIORITY	BSP_IRQPRIOMAX_MINUS(2)

#define		DAC_EV_UNDR			0x10000		// Underrun event for DAC interrupt callback

typedef enum
{
	DAC_ENONE		= 0,
	DAC_EFAIL		= 1,
	DAC_ENOTINIT	= 2,
	DAC_ETIMEOUT	= 3,
	DAC_EARG		= 4,
	DAC_ESTATE		= 5,

} dacStatus_t ;

#ifdef __cplusplus
extern "C" {
#endif

dacStatus_t 	dacInit				(uint32_t flags, uint32_t id) ;
dacStatus_t 	dacSetChannel		(uint32_t channel, uint32_t flags, uint32_t trigSrc) ;
dacStatus_t 	dacSetCallback		(uint32_t channel, dmaCallback_t cb, uintptr_t arg) ;
dacStatus_t 	dacDmaInit			(uint32_t channel, uint32_t flags, uint32_t length) ;
dacStatus_t 	dacDmaBuffer		(uint32_t channel, const void * buffer1, const void * buffer2) ;
dacStatus_t 	dacStart			(uint32_t channel) ;
dacStatus_t 	dacStop				(uint32_t channel) ;
dacStatus_t 	dacSwTrigger		(uint32_t channel) ;
dacStatus_t 	dacOuput			(uint32_t channel, uint32_t value)  ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// DACBASIC_H_
