/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dmabasic.c	DMA management

	When		Who	What
	23/04/18	ac	Creation


----------------------------------------------------------------------
*/

#if ! defined	DMABASIC_H_
#define DMABASIC_H_
//----------------------------------------------------------------------

#include	"stm32f1xx_ll_dma.h"

#define	DMA_TOSTREAMCHAN(stream,chan)	((uint8_t)(((stream) << 4) | (chan)))
#define	DMA_GETSTREAM(pDesc)			((pDesc)->streamChan >> 4)
#define	DMA_GETCHANNEL(pDesc)			((pDesc)->streamChan & 0x07u)
#define	DMA_GETDMA(dmaNum)				(((dmaNum) == 1) ? DMA1 : DMA2)	// Get pointer to DMa device from DMA number

#define	DMA_NONE	0

typedef struct
{
	uint8_t		dmaNum ;		//	DMA number 1 or 2, DMA_NONE => unused descriptor
	uint8_t		streamChan ;	//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
	uint8_t		dir ;			//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
	uint8_t		irqPrio ;		//	Interrupt priority

} dmaDesc_t ;

// Event of DMA IRQ callback function
#define	DMA_EV_COMPLETE		1		// Transfer complete
#define	DMA_EV_HCOMPLETE	2		// Transfer half complete
#define	DMA_EV_ERROR		3		// Transfer error

// Callback parameter dir is one of:
//	LL_DMA_DIRECTION_PERIPH_TO_MEMORY
//	LL_DMA_DIRECTION_MEMORY_TO_PERIPH
//	LL_DMA_DIRECTION_MEMORY_TO_MEMORY

// Prototype of DMA IRQ callback function
typedef void (* dmaCallback_t) (uint32_t event, uint32_t dir, uintptr_t arg) ;

//----------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

void	dmaSetIrq		(const dmaDesc_t * pDmaDesc) ;
void	dmaDisableIrq	(const dmaDesc_t * pDmaDesc) ;
void	dmaSetCallback	(const dmaDesc_t * pDmaDesc, dmaCallback_t fn, uintptr_t arg) ;

// Get stream register address
static inline DMA_Stream_TypeDef * dmaStreamAddress (DMA_TypeDef * pDma, uint32_t streamNum)
{
	  return (DMA_Stream_TypeDef *) ((uint32_t)pDma + STREAM_OFFSET_TAB[streamNum]) ;
}

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// DMABASIC_H_
