/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testdac.c	DAC basic API tests

	When		Who	What
	05/05/18	ac	Creation

----------------------------------------------------------------------
*/
#include	"aa.h"
#include	"aaprintf.h"
#include	"aalogmes.h"
#include	"dacbasic.h"
#include	"timbasic.h"
#include	"dmabasic.h"

#define		TIMER_DAC	6	// Id of timer descriptor used to trigger DAC
#define		TIMER_FREQ	50	// Timer frequency (Hz)

#define	MAX_DATA	8

const uint32_t	dacDataDual [MAX_DATA] =
{
	0x0FFF0000,
	0x0DB60249,
	0x0B6D0492,
	0x092406BD,
	0x06DB0924,
	0x04920B6D,
	0x02490DB6,
	0x00000FFF,
} ;

const uint16_t	dacData1 [MAX_DATA] =
{
	0x0000,
	0x0249,
	0x0492,
	0x06BD,
	0x0924,
	0x0B6D,
	0x0DB6,
	0x0FFF,
} ;

const uint16_t	dacData2 [MAX_DATA] =
{
	0x0FFF,
	0x0DB6,
	0x0B6D,
	0x0924,
	0x06BD,
	0x0492,
	0x0249,
	0x0000,
} ;

const uint16_t	dacData3 [MAX_DATA] =
{
	0x007FF,
	0x07FF,
	0x07FF,
	0x07FF,
	0x07FF,
	0x07FF,
	0x07FF,
	0x07FF,
} ;

const uint16_t	* dataArray [3] =
{
	dacData1,
	dacData2,
	dacData3,
};

uint32_t	dataIx ;	// dataArray index of next buffer for double buffer DMA

//----------------------------------------------------------------------
// DAC and DMA interrupts callback: for double buffer DMA management

void	dacDmaCallback (uint32_t event, uint32_t dir, uintptr_t arg)
{
	// DAC1 channel 1 interrupt management
	if (arg == DAC_CHAN1)
	{
		// Check if DMA complete
		if (event == DMA_EV_COMPLETE)
		{
			(void) dacDmaBuffer	 (DAC_CHAN1, (void *) dataArray[dataIx], NULL) ;
			dataIx++ ;
			if (dataIx == 3)
			{
				dataIx = 0 ;
			}
		}

		// Check if DMA underrun
		if (event == DAC_EV_UNDR)
		{
		}
	}

	// DAC1 channel 2 interrupt management
	if (arg == DAC_CHAN2)
	{
		// Check if DMA complete
		if (event == DMA_EV_COMPLETE)
		{
			(void) dacDmaBuffer	 (DAC_CHAN2, (void *) dataArray[dataIx], NULL) ;
			dataIx++ ;
			if (dataIx == 3)
			{
				dataIx = 0 ;
			}
		}

		// Check if DMA underrun
		if (event == DAC_EV_UNDR)
		{
		}
	}

	(void) dir ;
}

//----------------------------------------------------------------------
// Callback for one shot DMA on channel 2

void	dacDmaCbOneShot (uint32_t event, uint32_t dir, uintptr_t arg)
{
	if (event == DAC_EV_UNDR)
	{
		// Signal end of DMA to the task
		aaSignalSend ((aaTaskId_t) arg, 0x01) ;
		aaLogMes ("It one shot undr\n", 0, 0, 0, 0, 0) ;
	}
	(void) dir ;
}

//----------------------------------------------------------------------

void	dacTest ()
{
	dacStatus_t	res ;
	uint32_t	ix ;
	uint32_t	cc ;
	const timDesc_t		* pTimDesc ;

	cc = '2' ;

	switch (cc)
	{
		case '1':
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			//	Constant value, trigger software
			res = dacInit (DAC_MONO1, 11) ;
			if (res != DAC_ENONE)
			{
				aaPrintf ("dacInit error %u\n", res) ;
				return ;
			}

			res = dacSetChannel	(DAC_CHAN1, 0, LL_DAC_TRIG_SOFTWARE) ;
			if (res != DAC_ENONE)
			{
				aaPrintf ("dacSetChannel error %u\n", res) ;
				return ;
			}

			// Initialize DAC
			ix = 0 ;
			dacStart (DAC_CHAN1) ;

			while (1)
			{
				aaTaskDelay (5) ;
				dacOuput (DAC_CHAN1, dacData1 [ix++]) ;
				dacSwTrigger (DAC_CHAN1) ;
				if (ix == MAX_DATA)
				{
					ix = 0 ;
				}
			}
			dacStop (DAC_CHAN1) ;
			break ;

		case '2':
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			// Independent channel 2 with single shot DMA, interrupt, timer trigger

			// Configure DAC
			(void) dacInit       	(DAC_MONO2, 12) ;
			(void) dacSetChannel 	(DAC_CHAN2, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacDmaInit    	(DAC_CHAN2, DAC_DMA_SINGLE, MAX_DATA) ;
			(void) dacDmaBuffer	 	(DAC_CHAN2, (void *) dacData1, NULL) ;
			(void) dacSetCallback	(DAC_CHAN2, dacDmaCbOneShot, aaTaskSelfId ()) ;

			// Configure timer
			if (timGetDesc (TIMER_DAC, & pTimDesc) == TIM_EFAIL)
			{
				aaPrintf ("Timer DAC: Unknown descriptor id %u\n", TIMER_DAC) ;
				return ;
			}
			timTimeBaseInit     (pTimDesc, 0, TIMER_FREQ, 0) ;
			timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

			// WARNING: DAC stop drop the output signal to 0. So we don't use dacStop() here.
			while (1)
			{
				dacStart (DAC_CHAN2) ;
				timStart (pTimDesc) ;
				aaSignalWait (0x01, NULL, AA_SIGNAL_AND, AA_INFINITE) ;
				timStop (pTimDesc) ;
				aaTaskDelay (5) ;
			}
			break ;

		case '3':
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			// Independent channel 1 with double buffer DMA, no interrupt, timer trigger

			// Configure DAC
			(void) dacInit       	(DAC_MONO1, 11) ;
			(void) dacSetChannel 	(DAC_CHAN1, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacDmaInit    	(DAC_CHAN1, DAC_DMA_DOUBLE | DAC_DMA_NOIT, MAX_DATA) ;
			(void) dacDmaBuffer	 	(DAC_CHAN1, (void *) dacData1, (void *) dacData2) ;

			// Configure timer
			if (timGetDesc (TIMER_DAC, & pTimDesc) == TIM_EFAIL)
			{
				aaPrintf ("Timer DAC: Unknown descriptor id %u\n", TIMER_DAC) ;
				return ;
			}
			timTimeBaseInit     (pTimDesc, 0, TIMER_FREQ, 0) ;
			timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

			dacStart (DAC_CHAN1) ;
			timStart (pTimDesc) ;
			while (1)
			{
				aaTaskDelay (5) ;
			}
			dacStop (DAC_CHAN1) ;
			timStop (pTimDesc) ;
			break ;

		case '4':
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			// Independent channel 1 and 2 with circular DMA buffer, common timer trigger
			// Channel 1 use dacData1, channel 2 use dacData2
			// Interrupt enabled, no callback

			// Configure DAC
			(void) dacInit       (DAC_MONO1 | DAC_MONO2, 10) ;
			(void) dacSetChannel (DAC_CHAN1, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacSetChannel (DAC_CHAN2, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacDmaInit    (DAC_CHAN1, DAC_DMA_CIRCULAR, MAX_DATA) ;
			(void) dacDmaInit    (DAC_CHAN2, DAC_DMA_CIRCULAR, MAX_DATA) ;
			(void) dacDmaBuffer	 (DAC_CHAN1, (void *) dacData1, NULL) ;
			(void) dacDmaBuffer	 (DAC_CHAN2, (void *) dacData2, NULL) ;

			// Configure timer
			if (timGetDesc (TIMER_DAC, & pTimDesc) == TIM_EFAIL)
			{
				aaPrintf ("Timer DAC: Unknown descriptor id %u\n", TIMER_DAC) ;
				return ;
			}
			timTimeBaseInit     (pTimDesc, 0, TIMER_FREQ, 0) ;
			timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

			dacStart (DAC_CHAN1) ;
			dacStart (DAC_CHAN2) ;
			timStart (pTimDesc) ;
			while (1)
			{
				aaTaskDelay (5) ;
			}
			dacStop (DAC_CHAN1) ;
			dacStop (DAC_CHAN2) ;
			timStop (pTimDesc) ;
			break ;

		case '5':
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			// Independent channel 1 with DMA double buffer, timer trigger
			// Multi buffer management by interrupt callback

			// Configure DAC
			(void) dacInit			(DAC_MONO1, 11) ;
			(void) dacSetChannel	(DAC_CHAN1, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacDmaInit		(DAC_CHAN1, DAC_DMA_DOUBLE, MAX_DATA) ;
			(void) dacDmaBuffer		(DAC_CHAN1, (void *) dacData1, (void *) dacData2) ;
			(void) dacSetCallback	(DAC_CHAN1, dacDmaCallback, DAC_CHAN1) ;

			// Configure timer
			if (timGetDesc (TIMER_DAC, & pTimDesc) == TIM_EFAIL)
			{
				aaPrintf ("Timer DAC: Unknown descriptor id %u\n", TIMER_DAC) ;
				return ;
			}
			timTimeBaseInit     (pTimDesc, 0, TIMER_FREQ, 0) ;
			timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

			dataIx = 2 ;	// Next buffer
			dacStart (DAC_CHAN1) ;
			timStart (pTimDesc) ;
			while (1)
			{
				aaTaskDelay (5) ;
			}
			dacStop (DAC_CHAN1) ;
			timStop (pTimDesc) ;
			break ;

		case '6':
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			// Dual channels with circular DMA buffer, timer trigger

			// Configure DAC
			(void) dacInit			(DAC_DUAL, 11) ;
			(void) dacSetChannel	(DAC_CHAN1, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacSetChannel	(DAC_CHAN2, 0, LL_DAC_TRIG_EXT_TIM6_TRGO) ;
			(void) dacDmaInit		(DAC_CHAN1, DAC_DMA_CIRCULAR | DAC_DMA_NOIT, MAX_DATA) ;
			(void) dacDmaBuffer		(DAC_CHAN1, (void *) dacDataDual, NULL) ;

			// Configure timer
			if (timGetDesc (TIMER_DAC, & pTimDesc) == TIM_EFAIL)
			{
				aaPrintf ("Timer DAC: Unknown descriptor id %u\n", TIMER_DAC) ;
				return ;
			}
			timTimeBaseInit     (pTimDesc, 0, TIMER_FREQ, 0) ;
			timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

			dacStart (DAC_CHAN1) ;
			dacStart (DAC_CHAN2) ;
			timStart (pTimDesc) ;
			while (1)
			{
				aaTaskDelay (5) ;
			}
			dacStop (DAC_CHAN1) ;
			dacStop (DAC_CHAN2) ;
			timStop (pTimDesc) ;
			break ;

		default:
			break ;
	}
}

//----------------------------------------------------------------------
