/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	i2cC24xx.c	Master I2C library for 24Cxx EEPROM
				Allows to use 24C series of EEPROM with up to 2048 bytes

	When		Who	What
	09/04/18	ac	Creation

----------------------------------------------------------------------
*/

#if ! defined EE24CXX_H_
#define EE24CXX_H_
//----------------------------------------------------------------------
#include	"i2cbasic.h"

typedef enum
{
	EE24C01			= 0,	// 128  bytes
	EE24C02			= 1,	// 256  bytes
	EE24C04			= 2,	// 512  bytes
	EE24C08			= 3,	// 2014 bytes
	EE24C16			= 4,	// 2048 bytes

} ee24cType_t ;

//----------------------------------------------------------------------
extern	const uint32_t	ee24cPageDesc [5] ;

#define	EE24C_PAGEMASK(type)		((ee24cPageDesc[type] >> 16) & 0x00FFu)
#define	EE24C_PAGECOUNT(type)		((ee24cPageDesc[type] >> 8) & 0x00FFu)
#define	EE24C_PAGESIZE(type)		(ee24cPageDesc[type] & 0x00FFu)

//----------------------------------------------------------------------
//	EEPROM descriptor and functions

typedef struct
{
	const i2cDesc_t		* pI2cDesc ;
	ee24cType_t			eeType ;
	uint8_t				i2cAddress ;	// 7 bits left aligned
	uint8_t				header [2] ;

} ee24cDesc_t ;

i2cStatus_t	ee24cInit		(ee24cDesc_t * pDesc, uint32_t i2cId, ee24cType_t type, uint32_t i2cAddress) ;
i2cStatus_t	ee24cWrite		(ee24cDesc_t * pDesc, uint32_t offset, uint8_t * pBuffer, uint32_t count) ;
i2cStatus_t	ee24cRead		(ee24cDesc_t * pDesc, uint32_t offset, uint8_t * pBuffer, uint32_t count) ;

//----------------------------------------------------------------------
#endif	// EE24CXX_H_
