/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	i2cdescF4.c		STM32F429 LQFP144 I2C descriptors for i2cbasic.c

	When		Who	What
	01/03/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2020 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined I2CF1XX_H_
#define I2CF1XX_H_
//------------------------------------------------------------------------------

/*
I2C pin out for STM32F1xx.

		SDA		SCL

I2C1	PB7		PB6
		PB9		PB8		remap

I2C2	PB11	PB10
*/

//	These configurations can be modified and augmented by the user to comply with the application

STATIC	const i2cDesc_t		i2cArray [] =
{
	#if (WITH_I2C_1 == 1  &&  WITH_I2C_1R == 0)
	{
		1,						// Descriptor identifier
		RCC_APB1RSTR_I2C1RST_Pos,
		RCC_APB1ENR_I2C1EN_Pos,	// periph enable bit position
		{
			AA_GPIO_PORTB,		// port Sda
			9,					// pin  Sda
			AA_REMAP_I2C1+0,	// Without remap
			AA_GPIO_MODE_ALT_OD,
		},
		{
			AA_GPIO_PORTB,		// Port Scl
			8,					// pin  Scl
			0,					//
			AA_GPIO_MODE_ALT_OD,
		},
		I2C1,					// I2C_TypeDef*

		I2C1_EV_IRQn,
		I2C1_ER_IRQn,
		I2C_INT_PRIORITY,
	},
	#endif	// WITH_I2C_1

	#if (WITH_I2C_1 == 1  &&  WITH_I2C_1R == 1)
	{
		11,						// Descriptor identifier
		RCC_APB1RSTR_I2C1RST_Pos,
		RCC_APB1ENR_I2C1EN_Pos,	// periph enable bit position
		{
			AA_GPIO_PORTB,		// port Sda
			9,					// pin  Sda
			AA_REMAP_I2C1+1,	// With remap
			AA_GPIO_MODE_ALT_OD,
		},
		{
			AA_GPIO_PORTB,		// Port Scl
			8,					// pin  Scl
			0,					//
			AA_GPIO_MODE_ALT_OD,
		},
		I2C1,					// I2C_TypeDef*

		I2C1_EV_IRQn,
		I2C1_ER_IRQn,
		I2C_INT_PRIORITY,
	},
	#endif	// WITH_I2C_1

	#if (WITH_I2C_2 == 1)
	{
		2,						// Descriptor identifier
		RCC_APB1RSTR_I2C2RST_Pos,
		RCC_APB1ENR_I2C2EN_Pos,	// periphPos ;
		{
			AA_GPIO_PORTB,		// port Sda ;
			11,					// pin  Sda;
			0,					//
			AA_GPIO_MODE_ALT_OD,
		},
		{
			AA_GPIO_PORTB,		// port Scl;
			10,					// pin  Scl ;
			0,					//
			AA_GPIO_MODE_ALT_OD,
		},
		I2C2,					// I2C_TypeDef ;

		I2C2_EV_IRQn,
		I2C2_ER_IRQn,
		I2C_INT_PRIORITY,
	},
	#endif	// WITH_I2C_2

	// You can insert here other descriptors for alternate GPIO pins
	// Choose an unique descriptor identifier for every descriptor
} ;

//------------------------------------------------------------------------------
#endif	// I2CF1XX_H_
