/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	timF1cc.h		STM32F1xx Timer descriptors for timbasic.c

	When		Who	What
	09/03/18	ac	Creation

----------------------------------------------------------------------
*/
#if ! defined TIMF1xx_H_
#define TIMF1xx_H_


//------------------------------------------------------------------------------
/*	For STM32F103xB

TIM		AF	CH1		CH1N	CH2		CH2N	CH3		CH3N	CH4		BKIN	ETR
--------------------------------------------------------------------------------
TIM1	 	PA8		PB13	PA9		PB14	PA10	PB15	PA11	PB12	PA12
			   		PA7		    	PB0		    	PB1 	 				PA6

TIM2	 	PA0				PA1				PA2				PA3
			PA15			PB3				PB10			PB11

TIM3	 	PA6				PA7				PB0				PB1				PD2
			PC6				PC7				PC8				PC9
			PB4				PB5

TIM4	 	PB6				PB7				PB8				PB9				PE0

*/

//------------------------------------------------------------------------------
// One descriptors for each timer configuration

// Some timer manage more than 1 interrupt vector: TIM1 and TIM8
// The descriptor describe only 1 interrupt.
// If you need more than 1 interrupt vector, configure the other interrupts from your application code

// Count of timers in the processor
#define	MAX_COUNTER	8

const	timDesc_t	timDescArray [] =
{
	#if (WITH_TIM_1 == 1)
	{
		1,								// Descriptor identifier
		1,								// Timer number 1..14
		RCC_APB2RSTR_TIM1RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB2ENR_TIM1EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_ADVT | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_COMP_OUT, 	// Capabilities flags
		TIM1,							// Pointer to TIM registers
		TIM1_UP_TIM10_IRQn,
		{
			{	'A',	8,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch1
			{	'A',	9,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch2
			{	'A',	10,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch3
			{	'A',	11,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch4
			{	'B',	13,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch1N
			{	'B',	14,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch2N
			{	'A',	11,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM},		// Ch3N
			{	'A',	12,	0,	AA_GPIO_MODE_INPUT },		// ETR
			{	'B',	12,	0,	AA_GPIO_MODE_INPUT },		// BKIN
		},
	},
	#endif	// WITH_TIM1

	#if (WITH_TIM_2 == 1)
	{
		2,								// Descriptor identifier
		2,								// Timer number 1..14
		RCC_APB1RSTR_TIM2RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM2EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR, 	// Capabilities flags
		TIM2,							// Pointer to TIM registers
		TIM2_IRQn,
		{
			{	'A',	0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1
			{	'A',	1,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2
			{	'A',	2,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3
			{	'A',	3,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3N
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },		// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },		// BKIN
		},
	},
	#endif	// WITH_TIM2

	#if (WITH_TIM_3 == 1)
	{
		3,								// Descriptor identifier
		3,								// Timer number 1..14
		RCC_APB1RSTR_TIM3RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM3EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR, 	// Capabilities flags
		TIM3,							// Pointer to TIM registers
		TIM3_IRQn,
		{
			{	'A',	6,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1
			{	'A',	7,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2
			{	'A',	0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3
			{	'A',	1,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3N
			{	'D',	2,	0,	AA_GPIO_MODE_INPUT },		// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },		// BKIN
		},
	},
	#endif	// WITH_TIM3
	
	#if (WITH_TIM_4 == 1)
	{
		4,								// Descriptor identifier
		4,								// Timer number 1..14
		RCC_APB1RSTR_TIM4RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM4EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR, 	// Capabilities flags
		TIM4,							// Pointer to TIM registers
		TIM4_IRQn,
		{
			{	'B',	6,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch1
			{	'B',	7,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch2
			{	'B',	8,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch3
			{	'B',	9,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch3N
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// BKIN
		},
	},
	#endif	// WITH_TIM4
	
	#if (WITH_TIM_5 == 1)
	{
		5,								// Descriptor identifier
		5,								// Timer number 1..14
		RCC_APB1RSTR_TIM5RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM5EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA, 	// Capabilities flags
		TIM5,							// Pointer to TIM registers
		TIM5_IRQn,
		{
			{	'H',	10,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch1
			{	'H',	11,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch2
			{	'H',	12,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch3
			{	'I',	0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch3N
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// BKIN
		},
	},
	#endif	// WITH_TIM5

	#if (WITH_TIM_6 == 1)
	{
		6,								// Descriptor identifier
		6,								// Timer number 1..14
		RCC_APB1RSTR_TIM6RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM6EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_BASIC | TIM_CAP_DMA, 		// Capabilities flags
		TIM6,							// Pointer to TIM registers
		TIM6_DAC_IRQn,
		{
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch1
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch2
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch3
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM  },	// Ch3N
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// BKIN
		},
	},
	#endif	// WITH_TIM6

	#if (WITH_TIM_7 == 1)
	{
		7,								// Descriptor identifier
		7,								// Timer number 1..14
		RCC_APB1RSTR_TIM7RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM7EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_BASIC | TIM_CAP_DMA, 		// Capabilities flags
		TIM7,							// Pointer to TIM registers
		TIM7_IRQn,
		{
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch1
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch2
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch3
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// Ch3N
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },	// BKIN
		},
	},
	#endif	// WITH_TIM7

	#if (WITH_TIM_8 == 1)
	{
		8,								// Descriptor identifier
		8,								// Timer number 1..14
		RCC_APB1RSTR_TIM8RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM8EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_ADVT | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_COMP_OUT, 	// Capabilities flags
		TIM8,							// Pointer to TIM registers
		TIM8_UP_TIM13_IRQn,
		{
			{	'C',	6,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch1
			{	'A',	5,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch2
			{	'C',	7,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch3
			{	'B',	0,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch4
			{	'C',	8,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch1N
			{	'B',	1,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch2N
			{	'C',	9,	0,	AA_GPIO_MODE_ALT_PP | GPIO_SPEED_HIGH },	// Ch3N
			{	'I',	3,	0,	AA_GPIO_MODE_INPUT },	// ETR
			{	'I',	4,	0,	AA_GPIO_MODE_INPUT },	// BKIN
		},
	},
	#endif	// WITH_TIM8

	#if (WITH_TIM_2 == 1)
	{
		20,								// Descriptor identifier
		2,								// Timer number 1..14
		RCC_APB1RSTR_TIM2RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM2EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR | TIM_CAP_32_BITS, 	// Capabilities flags
		TIM2,							// Pointer to TIM registers
		TIM2_IRQn,
		{
			{	'A',	0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1
			{	0,		3,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2
			{	0,		10,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3
			{	0,		3,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3N
			{	0,		15,	0,	AA_GPIO_MODE_INPUT },		// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },		// BKIN
		},
	},
	#endif	// WITH_TIM2

	{
		41,								// Descriptor identifier
		4,								// Timer number 1..14
		RCC_APB1RSTR_TIM4RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
		RCC_APB1ENR_TIM4EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR, 	// Capabilities flags
		TIM4,							// Pointer to TIM registers
		TIM4_IRQn,
		{
			{	'B',	6,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1
			{	0,		3,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2
			{	0,		10,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3
			{	0,		3,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch4
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch1N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch2N
			{	0,		0,	0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// Ch3N
			{	0,		15,	0,	AA_GPIO_MODE_INPUT },		// ETR
			{	0,		0,	0,	AA_GPIO_MODE_INPUT },		// BKIN
		},
	},
} ;
//------------------------------------------------------------------------------
#endif	// TIMF1xx_H_
